"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheService = void 0;
const typedi_1 = require("typedi");
const config_1 = __importDefault(require("../config"));
const cache_manager_1 = require("cache-manager");
const GenericHelpers_1 = require("../GenericHelpers");
const n8n_workflow_1 = require("n8n-workflow");
let CacheService = exports.CacheService = class CacheService {
    async init() {
        if (!config_1.default.getEnv('cache.enabled')) {
            throw new Error('Cache is disabled');
        }
        const backend = config_1.default.getEnv('cache.backend');
        if (backend === 'redis' ||
            (backend === 'auto' && config_1.default.getEnv('executions.mode') === 'queue')) {
            const { redisInsStore } = await Promise.resolve().then(() => __importStar(require('cache-manager-ioredis-yet')));
            const { default: Redis } = await Promise.resolve().then(() => __importStar(require('ioredis')));
            let lastTimer = 0;
            let cumulativeTimeout = 0;
            const { host, port, username, password, db } = config_1.default.getEnv('queue.bull.redis');
            const clusterNodes = (0, GenericHelpers_1.getRedisClusterNodes)();
            const redisConnectionTimeoutLimit = config_1.default.getEnv('queue.bull.redis.timeoutThreshold');
            const usesRedisCluster = clusterNodes.length > 0;
            n8n_workflow_1.LoggerProxy.debug(usesRedisCluster
                ? `(Cache Service) Initialising Redis cluster connection with nodes: ${clusterNodes
                    .map((e) => `${e.host}:${e.port}`)
                    .join(',')}`
                : `(Cache Service) Initialising Redis client connection with host: ${host !== null && host !== void 0 ? host : 'localhost'} and port: ${port !== null && port !== void 0 ? port : '6379'}`);
            const sharedRedisOptions = {
                username,
                password,
                db,
                enableReadyCheck: false,
                maxRetriesPerRequest: null,
            };
            const redisClient = usesRedisCluster
                ? new Redis.Cluster(clusterNodes.map((node) => ({ host: node.host, port: node.port })), {
                    redisOptions: sharedRedisOptions,
                })
                : new Redis({
                    host,
                    port,
                    ...sharedRedisOptions,
                    retryStrategy: () => {
                        const now = Date.now();
                        if (now - lastTimer > 30000) {
                            lastTimer = now;
                            cumulativeTimeout = 0;
                        }
                        else {
                            cumulativeTimeout += now - lastTimer;
                            lastTimer = now;
                            if (cumulativeTimeout > redisConnectionTimeoutLimit) {
                                n8n_workflow_1.LoggerProxy.error(`Unable to connect to Redis after ${redisConnectionTimeoutLimit}. Exiting process.`);
                                process.exit(1);
                            }
                        }
                        return 500;
                    },
                });
            const redisStore = redisInsStore(redisClient, {
                ttl: config_1.default.getEnv('cache.redis.ttl'),
            });
            this.cache = await (0, cache_manager_1.caching)(redisStore);
        }
        else {
            const textEncoder = new TextEncoder();
            this.cache = await (0, cache_manager_1.caching)('memory', {
                ttl: config_1.default.getEnv('cache.memory.ttl'),
                maxSize: config_1.default.getEnv('cache.memory.maxSize'),
                sizeCalculation: (item) => {
                    return textEncoder.encode((0, n8n_workflow_1.jsonStringify)(item, { replaceCircularRefs: true })).length;
                },
            });
        }
    }
    async destroy() {
        if (this.cache) {
            await this.reset();
            this.cache = undefined;
        }
    }
    async getCache() {
        if (!this.cache) {
            await this.init();
        }
        return this.cache;
    }
    async get(key) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.get(key);
    }
    async set(key, value, ttl) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.set(key, value, ttl);
    }
    async delete(key) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.del(key);
    }
    async reset() {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.reset();
    }
    async keys() {
        var _a, _b;
        if (!this.cache) {
            await this.init();
        }
        return (_b = (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.keys()) !== null && _b !== void 0 ? _b : [];
    }
    async setMany(values, ttl) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.mset(values, ttl);
    }
    async getMany(keys) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.mget(...keys);
    }
    async deleteMany(keys) {
        var _a;
        if (!this.cache) {
            await this.init();
        }
        return (_a = this.cache) === null || _a === void 0 ? void 0 : _a.store.mdel(...keys);
    }
};
exports.CacheService = CacheService = __decorate([
    (0, typedi_1.Service)()
], CacheService);
//# sourceMappingURL=cache.service.js.map